#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <openssl/sha.h>
#include <openssl/rsa.h>
#include <openssl/bio.h>
#include <openssl/pem.h>
#include <openssl/err.h>


#define MAX_LINE_SIZE  1024
#define SIGNATURE_LENGTH 128

const char pub_pem[] = 
	"-----BEGIN PUBLIC KEY-----\r\n"
	"MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDGxBLkQeQkwShw7N6e/Z5IQ/bG\r\n"
	"QyYUpUcagys1nsAiTQ2roolsaiMY96RHgFsQ04MYWLPr6pbMOO//eskVvHO/IvjV\r\n"
	"ClLP5S+jLQ5MB0UCboWZO7591eAlYRHtIC/UagaEtwHVHNK/V1l7Tu8O0Ao5LbRt\r\n"
	"i4xcXQBwzh+iAB7FQwIDAQAB\r\n"
	"-----END PUBLIC KEY-----\r\n";

void printUsage(){
	printf("Usage: vali-xctrack <filename>\n");	
}

RSA *readPubKey() { 
    RSA *key; 
    BIO *bp; 
	
    bp = BIO_new_mem_buf((void *)pub_pem,sizeof(pub_pem));
    key = (RSA *)PEM_read_bio_RSA_PUBKEY(bp,NULL,NULL,NULL);
    BIO_free(bp); 
    
    return key; 
}

void exit_invalid() {
	printf("check FAILED\n");
	exit(1);
}

int from_hex(int c) {
	if (c >= '0' && c <= '9')
		return c-'0';
	else if (c >= 'A' && c <= 'F')
		return c-'A'+10;
	else {
		exit_invalid();
		return 0;		// make compiler happy
	}
}


int main(int arg, char * argv[]){
	FILE *filep;
	char line[MAX_LINE_SIZE];
	unsigned char sha[SHA_DIGEST_LENGTH];
	unsigned char signature[SIGNATURE_LENGTH];
	SHA_CTX sha_ctx;
	int sigPos = 0;
	
	if (arg != 2){
		printUsage();
		exit(1);
	}

	filep = fopen(argv[1], "rb");
	if (filep == NULL) {
		printf("File does not exist.\n");
		exit(1);
	}
	
	SHA1_Init(&sha_ctx);
	
	//read the igc File
	while (fgets(line, MAX_LINE_SIZE, filep) != NULL) {
		int len = strlen(line);
		while (len >= 1 && (line[len-1] == '\r' || line[len-1] == '\n')) len --;
		
		if (len > 0) {
			if (line[0] == 'G') {
				int i;
				for (i = 1; i+1 < len && sigPos < SIGNATURE_LENGTH; i += 2)
					signature[sigPos++] = from_hex(line[i])*16 + from_hex(line[i+1]);
				if (i != len)
					exit_invalid();
			}
			else if (line[0] !=  'L' || (len >= 4 && line[1] == 'X' && line[2] == 'C' && line[3] == 'T')) {
				SHA1_Update(&sha_ctx, line, len);
				SHA1_Update(&sha_ctx, "\r\n", 2);
			}
		}
	}
	
	if (sigPos != SIGNATURE_LENGTH) {
		exit_invalid();
	}
	
	SHA1_Final(sha, &sha_ctx);
	
	unsigned char buf[SIGNATURE_LENGTH];
	RSA_public_decrypt(SIGNATURE_LENGTH,signature,buf,readPubKey(),RSA_PKCS1_PADDING);
	if (memcmp(sha,buf,SHA_DIGEST_LENGTH) != 0)
		exit_invalid();
	printf("VALID\n");
		
	fclose(filep);
	
	return 0;
} 
